import cvxopt
from cvxopt import matrix
from cvxopt.solvers import qp
cvxopt.solvers.options['show_progress'] = False
import numpy as np


def TripletConvex(Xa_bags, Xb_bags, Xc_bags, n1, n2, basis, args):
    Xa = list([B.data() for B in Xa_bags])
    Xb = list([B.data() for B in Xb_bags])
    Xc = list([B.data() for B in Xc_bags])
    
    Xa = np.array([np.r_[1, basis(A)].T for A in Xa])
    Xb = np.array([np.r_[1, basis(B)].T for B in Xb])
    Xc = np.array([np.r_[1, basis(C)].T for C in Xc])
    
    N_a = len(Xa)
    N_b = len(Xb)
    N_c = len(Xc)    
    dim = Xa.shape[1]
    if args.lo == 'sq':
        return triplet_sq(Xa, N_a, Xb, N_b, Xc, N_c, n1, n2, dim, basis, args)
    elif args.lo == 'dh':
        return triplet_dh(Xa, N_a, Xb, N_b, Xc, N_c, n1, n2, dim, basis, args)

def triplet_sq(Xa, N_a, Xb, N_b, Xc, N_c, n1, n2, dim, basis, args):
    prior = args.prior
    prior_p = (1 + np.sqrt(1 - 4 * (1 - n1 / (n1 + n2)))) / 2
    prior_n = 1 - prior_p
    deno = 1 - prior_p * prior_n
    A = (prior_p*prior_p*prior_p + 2*prior_p*prior_p*prior_n) / deno
    B = (2*prior_p*prior_n*prior_n + prior_n*prior_n*prior_n) / deno
    a = A*A + prior_p*prior_p + prior_n*prior_n
    b = A*B + 2*prior_p*prior_n
    c = B*B + prior_p*prior_p + prior_n*prior_n
    coe = 1 / (a*c - b*b)
    A11 = prior*(c*prior_p-b*prior_n)*coe
    A12 = -(1-prior)*(a*prior_n-b*prior_p)*coe
    A21 = prior*(c*A-b*B)*coe
    A22 = -(1-prior)*(a*B-b*A)*coe
    A31 = -prior*(c*prior_n-b*prior_p)*coe
    A32 = (1-prior)*(a*prior_p-b*prior_n)*coe
    param = np.linalg.inv((A11-A12)/(2*N_a)* Xa.T.dot(Xa) + (A21-A22)/(2*N_b)* Xb.T.dot(Xb) + (A32-A31)/(2*N_c)* Xc.T.dot(Xc) + args.reg*np.eye(dim)).dot((A11+A12)/(2*N_a)*Xa.T.dot(np.ones(N_a)) + (A21+A22)/(2*N_b)*Xb.T.dot(np.ones(N_b)) - (A32+A31)/(2*N_c)*Xc.T.dot(np.ones(N_c)))
    w = param[1:]
    b = float(param[:1])
    clf = lambda X: w.T.dot(basis(X)) + b
    return clf

def triplet_dh(Xa, N_a, Xb, N_b, Xc, N_c, n1, n2, dim, basis, args):
    prior = args.prior
    prior_p = (1 + np.sqrt(1 - 4 * (1 - n1 / (n1 + n2)))) / 2
    prior_n = 1 - prior_p
    deno = 1 - prior_p * prior_n
    A = (prior_p*prior_p*prior_p + 2*prior_p*prior_p*prior_n) / deno
    B = (2*prior_p*prior_n*prior_n + prior_n*prior_n*prior_n) / deno
    a = A*A + prior_p*prior_p + prior_n*prior_n
    b = A*B + 2*prior_p*prior_n
    c = B*B + prior_p*prior_p + prior_n*prior_n
    coe = 1 / (a*c - b*b)
    A11 = prior*(c*prior_p-b*prior_n)*coe
    A12 = -(1-prior)*(a*prior_n-b*prior_p)*coe
    A21 = prior*(c*A-b*B)*coe
    A22 = -(1-prior)*(a*B-b*A)*coe
    A31 = -prior*(c*prior_n-b*prior_p)*coe
    A32 = (1-prior)*(a*prior_p-b*prior_n)*coe
    
    P = np.r_[
            np.c_[args.reg*np.eye(dim), np.zeros((dim, N_a)), np.zeros((dim, N_b)), np.zeros((dim, N_c))],
            np.c_[np.zeros((N_a, dim)), np.zeros((N_a, N_a)), np.zeros((N_a, N_b)), np.zeros((N_a, N_c))],
            np.c_[np.zeros((N_b, dim)), np.zeros((N_b, N_a)), np.zeros((N_b, N_b)), np.zeros((N_b, N_c))],
            np.c_[np.zeros((N_c, dim)), np.zeros((N_c, N_a)), np.zeros((N_c, N_b)), np.zeros((N_c, N_c))]
            ]
    
    
    q = np.r_[-A12/N_a*Xa.T.dot(np.ones((N_a,1)))-A22/N_b*Xb.T.dot(np.ones((N_b,1)))+A31/N_c*Xc.T.dot(np.ones((N_c,1))),
              (A11-A12)/(N_a)*np.ones((N_a,1)),
              (A21-A22)/(N_b)*np.ones((N_b,1)),
              (A32-A31)/(N_c)*np.ones((N_c,1))
              ]
    
    G = np.r_[
            np.c_[np.zeros((N_a, dim)), -np.eye(N_a), np.zeros((N_a, N_b)), np.zeros((N_a, N_c))],
            np.c_[-0.5*Xa,              -np.eye(N_a), np.zeros((N_a, N_b)), np.zeros((N_a, N_c))],
            np.c_[-Xa,                  -np.eye(N_a), np.zeros((N_a, N_b)), np.zeros((N_a, N_c))],
            np.c_[np.zeros((N_b, dim)), np.zeros((N_b, N_a)), -np.eye(N_b), np.zeros((N_b, N_c))],
            np.c_[-0.5*Xb,              np.zeros((N_b, N_a)), -np.eye(N_b), np.zeros((N_b, N_c))],
            np.c_[-Xb,                  np.zeros((N_b, N_a)), -np.eye(N_b), np.zeros((N_b, N_c))],
            np.c_[np.zeros((N_c, dim)), np.zeros((N_c, N_a)), np.zeros((N_c, N_b)), -np.eye(N_c)],
            np.c_[0.5*Xc,               np.zeros((N_c, N_a)), np.zeros((N_c, N_b)), -np.eye(N_c)],
            np.c_[Xc,                   np.zeros((N_c, N_a)), np.zeros((N_c, N_b)), -np.eye(N_c)]
            ]
    
    h = np.r_[np.zeros((N_a,1)), -0.5*np.ones((N_a,1)), np.zeros((N_a,1)), np.zeros((N_b,1)), -0.5*np.ones((N_b,1)), np.zeros((N_b,1)), 
    np.zeros((N_c,1)), -0.5*np.ones((N_c,1)), np.zeros((N_c,1))]


    result = qp(matrix(P), matrix(q), matrix(G), matrix(h))
    param = np.array(result['x'])
    w = param[1:dim]
    b = float(param[:1])
    clf = lambda X: w.T.dot(basis(X)) + b
    return clf    
            