import random
import torch.utils.data as data_utils
import numpy as np
import torchvision
import torch



def get_mnist():
    DOWNLOAD_MNIST = True
    train_data = torchvision.datasets.MNIST(root='./datasets',
                                            train=True,
                                            download=DOWNLOAD_MNIST,
                                            transform=torchvision.transforms.ToTensor())
    test_data = torchvision.datasets.MNIST(root='./datasets',
                                           train=False,
                                           download=DOWNLOAD_MNIST,
                                           transform=torchvision.transforms.ToTensor())
    x_train = torch.unsqueeze(train_data.train_data, dim=1).type(torch.FloatTensor) / 255.
    y_train = train_data.train_labels
    x_test = torch.unsqueeze(test_data.test_data, dim=1).type(torch.FloatTensor) / 255.
    y_test = test_data.test_labels
    x_train = x_train.reshape(-1, 784)
    x_test = x_test.reshape(-1, 784)
    data = torch.cat((x_train, x_test),dim =0)
    label = torch.cat((y_train, y_test),dim =0)
    return data, label
    
def get_Fashion_mnist():
    DOWNLOAD_F_MNIST = True
    train_data = torchvision.datasets.FashionMNIST(root='./datasets',train=True,download=DOWNLOAD_F_MNIST,transform=torchvision.transforms.ToTensor())
    test_data = torchvision.datasets.FashionMNIST(root='./datasets',train=False,download=DOWNLOAD_F_MNIST,transform=torchvision.transforms.ToTensor())
    x_train = torch.unsqueeze(train_data.train_data, dim=1).type(torch.FloatTensor) / 255.
    y_train = train_data.train_labels
    x_test = torch.unsqueeze(test_data.test_data, dim=1).type(torch.FloatTensor) / 255.
    y_test = test_data.test_labels
    x_train = x_train.reshape(-1, 784)
    x_test = x_test.reshape(-1, 784)
    data = torch.cat((x_train, x_test),dim =0)
    label = torch.cat((y_train, y_test),dim =0)

    return data, label


def get_kmnist():
    train_data = torchvision.datasets.KMNIST(root='./datasets', train=True, transform=torchvision.transforms.ToTensor(), download=True)
    test_data = torchvision.datasets.KMNIST(root='./datasets', train=False, transform=torchvision.transforms.ToTensor(), download=True)
    x_train = torch.unsqueeze(train_data.train_data, dim=1).type(torch.FloatTensor) / 255.
    y_train = train_data.train_labels
    x_test = torch.unsqueeze(test_data.test_data, dim=1).type(torch.FloatTensor) / 255.
    y_test = test_data.test_labels
    x_train = x_train.reshape(-1, 784)
    x_test = x_test.reshape(-1, 784)
    data = torch.cat((x_train, x_test),dim =0)
    label = torch.cat((y_train, y_test),dim =0)
    return data, label


def creat_bags(all_imgs, all_labels, target_number, num_bag,mean_bag_length,var_bag_length, seed):
    positive_ins = all_labels == target_number
    negative_ins = all_labels != target_number


    positive_idx = []
    negative_idx = []
    for i, p_ins in enumerate(positive_ins):
        if p_ins:
            positive_idx.append(i)

    for i, n_ins in enumerate(negative_ins):
        if n_ins:
            negative_idx.append(i)


    bags_list = []
    labels_list = []

    # positive
    for i in range(num_bag // 2):
        bag_length = int(np.random.RandomState(seed).normal(mean_bag_length, var_bag_length, 1))
        positive_num = abs(bag_length - mean_bag_length)
        if positive_num == 0:
            positive_num = mean_bag_length//5

        if bag_length < 1:
            bag_length = 1

        indices = random.sample(negative_idx, bag_length - positive_num)
        try:
            t = random.sample(positive_idx, positive_num)
        except ValueError:
            print('No more positive instances !')
            break
        positive_idx = list(set(positive_idx) - set(t))
        for i in t:
            indices.append(i)

        indices = torch.LongTensor(indices)

        labels_in_bag = all_labels[indices]
        if target_number in labels_in_bag:
            labels_in_bag = 1
        else:
            labels_in_bag = -1
        bags_list.append(all_imgs[indices])
        labels_list.append(labels_in_bag)

    # negative
    for i in range(num_bag // 2):
        bag_length = int(np.random.RandomState(seed).normal(mean_bag_length, var_bag_length, 1))
        if bag_length < 1:
            bag_length = 1
        try:
            indices = random.sample(negative_idx, bag_length)
        except ValueError:
            print('No more negative instances !')
            break
        negative_idx = list(set(negative_idx) - set(t))
        
        indices = torch.LongTensor(indices)
        labels_in_bag = all_labels[indices]
        if target_number in labels_in_bag:
            labels_in_bag = 1
        else:
            labels_in_bag = -1
        bags_list.append(all_imgs[indices])
        labels_list.append(labels_in_bag)    
    ordinary_data = []
    for i in range(len(bags_list)):
        for j in range(len(bags_list[i])):
            x = np.array(bags_list[i][j])
            y = labels_list[i]
            index= i
            ordinary_data.append({'x': x, 'y': int(y), 'bag_id': int(index)})
    dim = 784
    return ordinary_data, dim
        