
import numpy as np
from utils_algo import minimax_basis, check_accuracy
from convert_to_triplet_bags import convert_to_triplet_bags
from algorithms import TripletConvex
from utils_data import data_processing, Bag
from datasets import creat_bags, get_mnist, get_kmnist, get_Fashion_mnist
import argparse
import torch
parser = argparse.ArgumentParser()
parser.add_argument('-ds', help='specify a dataset', default='mnist', type=str, choices=['kmnist','mnist','fashion','musk1'],required=False)
parser.add_argument('-me', help='specify a approach', default='miltcvx', choices=['miltcvx'], type=str, required=False)
parser.add_argument('-lo', help='specify a loss', default='sq', choices=['sq','dh'], type=str, required=False)
parser.add_argument('-prior', help='class prior (the ratio of positive bags)', default=0.7, type=float, metavar='[0-1]')
parser.add_argument('-seed', help = 'Random seed', default=0, type=int, required=False)
parser.add_argument('-M', help = 'number of total training bags', default=180, type=int, required=False)
parser.add_argument('-degree', help = 'degree of polynomial kernel', default=1, type=int, required=False)
parser.add_argument('-reg', help = 'regularization parameter', default=1e+3, type=float, required=False)
parser.add_argument('-target', help = 'target to creative multi-instance dataset', default=0, type=int, required=False)
args = parser.parse_args()
torch.manual_seed(args.seed); torch.cuda.manual_seed_all(args.seed);np.random.seed(args.seed)
if args.ds == 'musk1':
    num_bags = 92*10
elif args.ds == 'musk2':
    num_bags = 102*10
elif args.ds == 'elephant':
    num_bags = 100*10
elif args.ds == 'fox':
    num_bags = 100*10
elif args.ds == 'tiger':
    num_bags = 100*10
elif args.ds == 'component':
    num_bags = 3130
elif args.ds == 'function':
    num_bags = 5242
elif args.ds == 'process':
    num_bags = 11718
elif args.ds == 'mnist' or args.ds == 'fashion' or args.ds == 'kmnist':
    num_bags = 3500



if args.ds == 'kmnist':
    all_data, all_label = get_kmnist()
    ordinary_data, dim= creat_bags(all_data,all_label,target_number=args.target,num_bag=num_bags,mean_bag_length=10,var_bag_length=2,seed=args.seed)
elif args.ds == 'fashion':
    all_data, all_label = get_Fashion_mnist()
    ordinary_data, dim= creat_bags(all_data,all_label,target_number=args.target,num_bag=num_bags,mean_bag_length=10,var_bag_length=2,seed=args.seed)
elif args.ds == 'mnist':    
    all_data, all_label = get_mnist()
    ordinary_data, dim= creat_bags(all_data,all_label,target_number=args.target,num_bag=num_bags,mean_bag_length=10,var_bag_length=2,seed=args.seed)
else:
    ordinary_data, dim= data_processing(args.ds)
    
    
ordinary_bags = [Bag(list(map(lambda X: {'data': X['x'], 'label': X['y']}, list(filter(lambda X: X['bag_id'] == i, ordinary_data)))))
    for i in range(num_bags)] # remove bag_id, and put into Bag()

Xa_bags, Xb_bags, Xc_bags, test_bags, n1, n2 = convert_to_triplet_bags(ordinary_bags, args.M, args)


train_bags_with_label = Xa_bags + Xb_bags + Xc_bags # train_bags with label
train_bags_without_label = list([B.data() for B in train_bags_with_label])
basis = minimax_basis(train_bags_without_label, args.degree)
model = TripletConvex(Xa_bags, Xb_bags, Xc_bags, n1, n2, basis, args)
acc = check_accuracy(model, test_bags, aucplot=0)
print('method: ', args.me, 'dataset: ', args.ds, 'seed: ', args.seed)
print('loss:', args.lo, 'reg: ', args.reg)
print("Accuracy of {}: {}".format(args.me, acc))

